# taskman

[![standard-readme compliant](https://img.shields.io/badge/standard--readme-OK-green.svg?style=flat-square)](https://github.com/RichardLitt/standard-readme)

TODO: Put more badges here.

Taskman is a task management or todo list software manageable both in command-line and browser.

It attempts to be, but not necessarily it is at the moment:

- portable, at least in C and javascript sense;
- minimal;
- low CPU & memory footprint.

Right now taskman consists of:

- `libtm` - a C library to manipulate tasks. Adding, removing, modifying, querying tasks.
  Backed by [SQLite](https://www.sqlite.org/index.html).
- `tm` - a command-line tool to access `libtm`. Adding, removing, modifying, querying tasks.
- `tms` - a HTTP server, backed by [Mongoose](https://github.com/cesanta/mongoose) (HTTP server),
  [jansson](https://jansson.readthedocs.io/) (JSON)
  and [hescape](https://github.com/k0kubun/hescape) (HTML escaping). Serves:

  - RPC endpoints for adding, removing, modifying, querying tasks;
  - Very basic website with adding, querying tasks, viewing and editing a task.
    Works with javascript turned off, and enabled with [history API](https://developer.mozilla.org/en-US/docs/Web/API/History_API).

Uninteresting stuff:

- Builds with Meson, and thus Ninja.
- Builds & runs on my Debian GNU/Linux, should not be Windows-ready.
- Right now tasks consist of:

  - No surprise here: ID;
  - Properties: name, description, status, date of creation, date of last update;
  - Initially status defaults to "new";
  - History of properties changes - value and date of change.

- `libtm` has a C test suite that links directly, powered by [Criterion](https://github.com/Snaipe/Criterion).
- `tm` has a C test suite that runs the tool and verifies output - Criterion too.
- About `tms`:

  - C test suite that runs the tool, verifies outputs and verifies HTTP responses. Criterion.
  - Javascript files are tested by [jest](https://github.com/facebook/jest/).
  - Javascript files and dependencies are "compiled" against [Browserify](http://browserify.org/),
    [babelify](https://github.com/babel/babelify). Because I like `require()` and I'd like
    javascripts to work on most platforms.
  - Some static content is generated by `tpl-lib` and `tpl-to-c` combination, which employ
    basic templating engine:

    - `tpl-to-c` converts template to `.h` and `.c` file with method that generates the content;
    - `tpl-lib` interfaces the method to `tms`;
    - Both have C test suite of course.

  - Other static content is just converted to C files via internal tool `file-to-c`,
     and included in the binary. Has C test suite of course. Criterion?

- Desktop browser end-to-end tests using [puppeteer](https://github.com/GoogleChrome/puppeteer) and `jest`.
  The tests spawn `tms`

- Lots of continuous integration scripts to ease work:

  - `ci/build-gcc.sh` builds and runs tests against `gcc`;
  - `ci/build-clang.sh` same for `clang`;
  - `ci/build-js.sh` same for javascripts;
  - `ci/analysis.sh` runs static code analysis tools against everything;
  - `ci/e2e.sh` runs end-to-end-tests;
  - `ci/coverage.sh` prints code coverage for native test suites;
  - `ci/leaks.sh` checks C tests for leaks.

## Table of Contents

- [Security](#security)
- [Install](#install)
- [Usage](#usage)
- [API](#api)
- [Maintainers](#maintainers)
- [Contribute](#contribute)
- [License](#license)

## Security

These security measures are NOT present in taskman:

- accounts, passwords, cookies, sessions;
- database encryptions;
- obfuscations of code;
- SSL/TLS/HTTPS - server should be setup behind a reverse proxy;
- cookies.

Consider this WIP and do not leave it exposed to Internet.
Also GDPR impact is not known nor analyzed at the moment, so consider taskman to be non-compliant.

## Install

Have:

- [Meson](http://mesonbuild.com/);
- [Node.js](https://nodejs.org/en/), and thus [npm](https://www.npmjs.com/).
  Required only for `tms`.

All other dependencies will be automatically downloaded.

```bash
meson build && cd build && ninja install
```

## Usage

### Command-line

Add tasks:

```bash
tm a "Buy milk"
tm add "Buy milk"
tm a "Buy milk" description "Description" status "in-review"
```

List/query tasks:

```bash
tm q
tm query

# By ID
tm q i 1
tm q id 1

# By name
tm q n milk
tm q name milk

# By description
tm q n milk
tm q description milk

# By status
tm q s milk
tm q status milk
```

Or... getting a single task:

```bash
tm p 1
tm print 1
```

Modify tasks. Use same filters as for `tm q`:

```bash
tm m i 1 -- n "Buy beer" s "open"
tm modify n milk -- n "Buy beer" s "open"
```

Remove tasks. Use same filters as for `tm q` and `tm m`:

```bash
tm r i 1
rm remove n beer
```

### Server

Start server on default IP 127.0.0.1 and port 3805:

```bash
tms
tms -i 127.0.0.1 -p 3805
```

## API-S

### Library

TODO. Consult `libtm/test/*.c` tests for usage.

### Server REST endpoints

Adding tasks:

```http
POST /api/add
Content-Type: application/json

{
    "name": "milk",
    "description": "description",
    "status": "status"
}

HTTP/1.1 200 OK
Content-Length: 0

```

Listing/querying tasks:

```http
POST /api/query
Content-Type: application/json

{
    "filters": { //OPTIONAL
        "id-equals": 1, //OPTIONAL
        "name-contains": "milk", //OPTIONAL
        "description-contains": "milk", //OPTIONAL
        "status-contains": "open", //OPTIONAL
        "date-created-after": "1970-01-01 23:59:59", //OPTIONAL
        "date-created-before": "1970-01-01 23:59:59", //OPTIONAL
        "date-updated-after": "1970-01-01 23:59:59", //OPTIONAL
        "date-updated-before": "1970-01-01 23:59:59" //OPTIONAL
    },
    "opts": { //OPTIONAL
        "show-column-description": true, //OPTIONAL
        "show-column-status": true //OPTIONAL
    }
}

HTTP/1.1 200 OK
Content-Type: application/json

{
    "columns": [], // Can contain 'description', 'status'
    "items": [
        {"id": 1, "name": "Buy milk"}, // , "description":"...", "status":"..."
        ...
    ]
}
```

Or...

```http
GET /api/query
Content-Type: application/json


HTTP/1.1 200 OK
Content-Type: application/json

{
    ...
}
```

Or... getting a single task

```http
GET /api/task/1
Content-Type: application/json


HTTP/1.1 200 OK
Content-Type: application/json

{
    "name": "Buy milk",
    "description": "Goat milk",
    "status": "open",
    "date-created": 12345,
    "date-updated": 23456,
    "history": [
        {"type": "change", "name": "name", "value": "Buy malk", "new-value": "Buy mlik", date: 1234},
        {"type": "add", "name": "name", "value": "Buy malk", date: 1200},
        ..
    ]
}
```

Modifying tasks:

```http
POST /api/modify
Content-Type: application/json

{
    "filters":
    {
        "id-equals": 1, //OPTIONAL
        "name-contains": "milk", //OPTIONAL
        "description-contains": "milk", //OPTIONAL
        "status-contains": "open", //OPTIONAL
        "date-created-after": "1970-01-01 23:59:59", //OPTIONAL
        "date-created-before": "1970-01-01 23:59:59", //OPTIONAL
        "date-updated-after": "1970-01-01 23:59:59", //OPTIONAL
        "date-updated-before": "1970-01-01 23:59:59" //OPTIONAL
    },
    "modifiers":
    {
        "name": "Buy beer",
        "description": "Buy beer",
        "status": "done"
    }
}

HTTP/1.1 200 OK
Content-Length: 0

```

Removing tasks:

```http
POST /api/remove
Content-Type: application/json

{
    "id-equals": 1, //OPTIONAL
    "name-contains": "milk", //OPTIONAL
    "description-contains": "milk", //OPTIONAL
    "status-contains": "milk", //OPTIONAL
    "date-created-after": "1970-01-01 23:59:59", //OPTIONAL
    "date-created-before": "1970-01-01 23:59:59", //OPTIONAL
    "date-updated-after": "1970-01-01 23:59:59", //OPTIONAL
    "date-updated-before": "1970-01-01 23:59:59" //OPTIONAL
}

HTTP/1.1 200 OK
Content-Length: 0

```

## Tests and other continuous integration stuff

Have:

- tests:

  - [Criterion](https://github.com/Snaipe/Criterion) - "native";
  - [jest](https://github.com/facebook/jest/) - javascript,
  - [puppeteer](https://github.com/GoogleChrome/puppeteer) - javascript (e2e);

- code analysis:

  - [Cppcheck](http://cppcheck.sourceforge.net/) - C,
  - [Vera++](https://bitbucket.org/verateam/vera/wiki/Home) - C,
  - [pmccabe](http://packages.debian.org/sid/pmccabe) - C,
  - [ESLint](https://eslint.org/) - javascript,
  - [HTMLHint](https://github.com/thedaviddias/HTMLHint) - html files,
  - [markdownlint-cli](https://www.npmjs.com/package/markdownlint-cli) - this README and other markdown files,
  - [Alex](http://alexjs.com/) - this README and other markdown files,
  - [markdown-spellcheck](https://www.npmjs.com/package/markdown-spellcheck) - this README and other markdown files.

- C code coverage:

  - [Gcovr](https://gcovr.com/).

- C leak checks:

  - [Valgrind](http://valgrind.org/).

## Maintainers

[@tsieprawski](https://bitbucket.org/tsieprawski)

## Contribute

PRs accepted.

Small note: If editing the README, please conform to the
[standard-readme](https://github.com/RichardLitt/standard-readme) specification.

## License

MIT © 2018 Tomasz Sieprawski
