/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <hescape.h>

#include <tpl.h>

// cppcheck-suppress unusedFunction
int tpl_tag(char **pout, unsigned int *plen, tpl_tag_f tag_f, void *userdata, char *tag)
{
    if (0 == tag_f || 0 == (tag_f)(pout, plen, userdata, tag))
    {
        fprintf(stderr, "tpl_tag() failed for tag \"%s\"", tag);
        return 0;
    }
    return 1;
}

// cppcheck-suppress unusedFunction
int tpl_enter(char **pout, unsigned int *plen, tpl_enter_f enter_f,
    void *userdata, char *name)
{
    if (0 == enter_f || 0 == (enter_f)(pout, plen, userdata, name))
    {
        return 0;
    }
    return 1;
}

// cppcheck-suppress unusedFunction
int tpl_next(char **pout, unsigned int *plen, tpl_next_f next_f,
    void *userdata)
{
    if (0 == next_f || 0 == (next_f)(pout, plen, userdata))
    {
        return 0;
    }
    return 1;
}

// cppcheck-suppress unusedFunction
int tpl_leave(char **pout, unsigned int *plen, tpl_leave_f leave_f,
    void *userdata)
{
    if (0 == leave_f)
    {
        return 0;
    }
    (leave_f)(pout, plen, userdata);
    return 1;
}

int tpl_appende(char **pout, unsigned int *plen, char *txt, unsigned int len)
{
    char *ntxt = 0;
    unsigned int nlen;
    int ret;

    if (0 == len)
    {
        return 1;
    }

    nlen = (unsigned int)hesc_escape_html((uint8_t **)&ntxt, (uint8_t *)txt, len);
    ret = tpl_append(pout, plen, ntxt, nlen);
    if (nlen > len)
    {
        free(ntxt);
    }
    return ret;
}

int tpl_append(char **pout, unsigned int *plen, char *txt, unsigned int len)
{
    char *nout = 0;

    if (0 == len)
    {
        return 1;
    }

    nout = (char *)realloc(*pout, (*plen) + len + 1);
    if (0 == nout)
    {
        return 0;
    }

    *pout = nout;
    memcpy((*pout) + (*plen), txt, len);
    (*plen) += len;
    (*pout)[*plen] = 0;
    return 1;
}
