/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <criterion/criterion.h>

#include <tpl.h>

TestSuite(tpl);

Test(tpl, append_from_0_to_0)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(1, tpl_append(&out, &len, "", 0));
    cr_expect_eq(0, out);
    cr_expect_eq(0, len);
}

Test(tpl, append_from_0_to_5)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(1, tpl_append(&out, &len, "rower", 5));
    cr_expect_str_eq("rower", out);
    cr_expect_eq(5, len);
    free(out);
}

Test(tpl, append_from_5_to_5)
{
    char *out = strdup("beton");
    unsigned int len = 5;

    cr_expect_eq(1, tpl_append(&out, &len, "", 0));
    cr_expect_str_eq("beton", out);
    cr_expect_eq(5, len);
    free(out);
}

Test(tpl, append_from_5_to_10)
{
    char *out = strdup("beton");
    unsigned int len = 5;

    cr_expect_eq(1, tpl_append(&out, &len, "rower", 5));
    cr_expect_str_eq("betonrower", out);
    cr_expect_eq(10, len);
    free(out);
}

Test(tpl, tag_no_callback)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(0, tpl_tag(&out, &len, 0, 0, "tag"));
    cr_expect_eq(0, out);
    cr_expect_eq(0, len);
}

static int tag_fail(char **pout, unsigned int *plen, void *userdata, char *tag)
{
    cr_expect_str_eq("tag", tag);
    return 0;
}

Test(tpl, tag_fail)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(0, tpl_tag(&out, &len, tag_fail, 0, "tag"));
    cr_expect_eq(0, out);
    cr_expect_eq(0, len);
}


static int tag_ok(char **pout, unsigned int *plen, void *userdata, char *tag)
{
    cr_expect_str_eq("tag", tag);
    return tpl_append(pout, plen, "rower", 5);
}

Test(tpl, tag_ok)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(1, tpl_tag(&out, &len, tag_ok, 0, "tag"));
    cr_expect_str_eq("rower", out);
    cr_expect_eq(5, len);
    free(out);
}

Test(tpl, enter_no_callback)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(0, tpl_enter(&out, &len, 0, 0, "tag"));
    cr_expect_eq(0, out);
    cr_expect_eq(0, len);
}

static int enter_fail(char **pout, unsigned int *plen, void *userdata, char *name)
{
    cr_expect_str_eq("tag", name);
    return 0;
}

Test(tpl, enter_fail)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(0, tpl_enter(&out, &len, enter_fail, 0, "tag"));
    cr_expect_eq(0, out);
    cr_expect_eq(0, len);
}


static int enter_ok(char **pout, unsigned int *plen, void *userdata, char *name)
{
    cr_expect_str_eq("tag", name);
    return tpl_append(pout, plen, "rower", 5);
}

Test(tpl, enter_ok)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(1, tpl_enter(&out, &len, enter_ok, 0, "tag"));
    cr_expect_str_eq("rower", out);
    cr_expect_eq(5, len);
    free(out);
}

Test(tpl, next_no_callback)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(0, tpl_next(&out, &len, 0, 0));
    cr_expect_eq(0, out);
    cr_expect_eq(0, len);
}

static int next_fail(char **pout, unsigned int *plen, void *userdata)
{
    return 0;
}

Test(tpl, next_fail)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(0, tpl_next(&out, &len, next_fail, 0));
    cr_expect_eq(0, out);
    cr_expect_eq(0, len);
}


static int next_ok(char **pout, unsigned int *plen, void *userdata)
{
    return tpl_append(pout, plen, "rower", 5);
}

Test(tpl, next_ok)
{
    char *out = 0;
    unsigned int len = 0;

    cr_expect_eq(1, tpl_next(&out, &len, next_ok, 0));
    cr_expect_str_eq("rower", out);
    cr_expect_eq(5, len);
    free(out);
}

Test(tpl, leave_no_callback)
{
    char *out = 0;
    unsigned int len = 0;

    tpl_leave(&out, &len, 0, 0);
    cr_expect_eq(0, out);
    cr_expect_eq(0, len);
}

static void leave_ok(char **pout, unsigned int *plen, void *userdata)
{
    tpl_append(pout, plen, "rower", 5);
}

Test(tpl, leave_ok)
{
    char *out = 0;
    unsigned int len = 0;

    tpl_leave(&out, &len, leave_ok, 0);
    cr_expect_str_eq("rower", out);
    cr_expect_eq(5, len);
    free(out);
}

