#!/usr/bin/python3
"""
    Runs meson tests against valgrind, with trace children on
    and exits 1 if any child leaked.
"""

from os import environ
import re
import subprocess

from glob import glob

if __name__ == '__main__':
    ARGS = ['meson', 'test',
            '--wrap=valgrind ' + \
            '--log-file=memcheck.%p.%n.log ' + \
            '--leak-check=full ' + \
            '--show-leak-kinds=all ' + \
            '--leak-resolution=high ' + \
            '--num-callers=20 ' + \
            '--show-reachable=yes ' + \
            '--trace-children=yes',
            '--suite', 'native'
           ]
    RET = subprocess.Popen(ARGS, cwd=environ['MESON_BUILD_ROOT']).wait()
    RESULTS = glob('{}/memcheck.*.log'.format(environ['MESON_BUILD_ROOT']))
    ANY_FAILURE = False
    PATTERN = re.compile("ERROR SUMMARY: [1-9]")
    for afile in RESULTS:
        found = any([True for line in open(afile).readlines() if PATTERN.search(line)])
        if found:
            ANY_FAILURE = True
            print("# {}".format(afile))
            for line in open(afile).readlines():
                print(line)
            print()

    exit(ANY_FAILURE)
