/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/
#ifndef HAVE_TM
#define HAVE_TM

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

typedef struct tm_lib tm_lib;
typedef struct tm_query tm_query;
typedef struct tm_storage tm_storage;

typedef struct tm_task tm_task;
typedef struct tm_task_diff tm_task_diff;

struct tm_storage;

int tm_storage_sqlite(char *file, tm_storage **);

struct tm_task;

int tm_task_new(tm_task **);
int tm_task_free(tm_task **);

int tm_task_get_id(tm_task *, unsigned long *);
int tm_task_set_id(tm_task *, unsigned long);

int tm_task_get_date_created(tm_task *, time_t *);
int tm_task_get_date_updated(tm_task *, time_t *);

int tm_task_get_name(tm_task *, char **);
int tm_task_set_name(tm_task *, char *);
int tm_task_get_description(tm_task *, char **);
int tm_task_set_description(tm_task *, char *);
int tm_task_get_status(tm_task *, char **);
int tm_task_set_status(tm_task *, char *);

int tm_task_arr_free(tm_task ***, unsigned int);
int tm_task_arr_set_name(tm_task **, unsigned int, char *);
int tm_task_arr_set_description(tm_task **, unsigned int, char *);
int tm_task_arr_set_status(tm_task **, unsigned int, char *);

struct tm_task_diff;

int tm_task_diff_new(tm_task_diff **);
int tm_task_diff_free(tm_task_diff **);

int tm_task_diff_get_id(tm_task_diff *, unsigned long *);
int tm_task_diff_set_id(tm_task_diff *, unsigned long);
int tm_task_diff_get_date(tm_task_diff *, time_t *);
int tm_task_diff_get_task(tm_task_diff *, unsigned long *);
int tm_task_diff_set_task(tm_task_diff *, unsigned long);
int tm_task_diff_get_name(tm_task_diff *, char **);
int tm_task_diff_set_name(tm_task_diff *, char *);
int tm_task_diff_get_value(tm_task_diff *, char **);
int tm_task_diff_set_value(tm_task_diff *, char *);

int tm_task_diff_arr_free(tm_task_diff ***, unsigned int);

struct tm_query;

int tm_query_id_equals(tm_lib *, unsigned long, tm_query **);
int tm_query_name_contains(tm_lib *, char *, tm_query **);
int tm_query_description_contains(tm_lib *, char *, tm_query **);
int tm_query_status_contains(tm_lib *, char *, tm_query **);
int tm_query_date_created_after(tm_lib *, time_t, tm_query **);
int tm_query_date_created_before(tm_lib *, time_t, tm_query **);
int tm_query_date_updated_after(tm_lib *, time_t, tm_query **);
int tm_query_date_updated_before(tm_lib *, time_t, tm_query **);
int tm_query_and(tm_lib *, unsigned int, tm_query **, tm_query **);
int tm_query_free(tm_query **);

struct tm;

int tm_new(tm_storage *, tm_lib **);
int tm_free(tm_lib **);

int tm_get_task(tm_lib *, unsigned long, tm_task **);
int tm_add_task(tm_lib *, tm_task *);
int tm_modify_task(tm_lib *, tm_task *);
int tm_remove_task(tm_lib *, tm_task *);

int tm_get_task_diffs(tm_lib *, tm_task *, tm_task_diff ***, unsigned int *);

int tm_run_query(tm_lib *, tm_query *, tm_task ***, unsigned int *);
int tm_get_tasks(tm_lib *, tm_query *, tm_task ***, unsigned int *);
int tm_add_tasks(tm_lib *, tm_task **, unsigned int);
int tm_modify_tasks(tm_lib *, tm_task **, unsigned int);
int tm_remove_tasks(tm_lib *, tm_task **, unsigned int);

void tm_compact_sprintui(char **pthis, unsigned int *plen, unsigned int value);
void tm_compact_sprintul(char **pthis, unsigned int *plen, unsigned long value);
void tm_compact_sprintp(char **pthis, unsigned int *plen, void *value);
void tm_compact_sprints(char **pthis, unsigned int *plen, char *value);

#endif
