/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#ifndef HAVE_PRV_STORAGE
#define HAVE_PRV_STORAGE

#include <tm.h>

typedef void (*tm_storage_free_f)(tm_storage *);
typedef int (*tm_storage_get_tasks_f)(tm_storage *, tm_query *, tm_task ***, unsigned int *);
typedef int (*tm_storage_add_tasks_f)(tm_storage *, tm_task **tasks, unsigned int);
typedef int (*tm_storage_modify_tasks_f)(tm_storage *, tm_task **tasks, unsigned int);
typedef int (*tm_storage_remove_tasks_f)(tm_storage *, tm_task **tasks, unsigned int);
typedef int (*tm_storage_get_task_f)(tm_storage *, unsigned long, tm_task **);
typedef int (*tm_storage_add_task_f)(tm_storage *, tm_task *);
typedef int (*tm_storage_modify_task_f)(tm_storage *, tm_task *);
typedef int (*tm_storage_remove_task_f)(tm_storage *, tm_task *);
typedef int (*tm_storage_get_task_diffs_f)(tm_storage *, tm_task *, tm_task_diff ***,
    unsigned int *);
typedef int (*tm_storage_query_id_equals_f)(tm_storage *, unsigned long, tm_query **);
typedef int (*tm_storage_query_name_contains_f)(tm_storage *, char *, tm_query **);
typedef int (*tm_storage_query_description_contains_f)(tm_storage *, char *, tm_query **);
typedef int (*tm_storage_query_status_contains_f)(tm_storage *, char *, tm_query **);
typedef int (*tm_storage_query_date_created_after_f)(tm_storage *, time_t, tm_query **);
typedef int (*tm_storage_query_date_created_before_f)(tm_storage *, time_t, tm_query **);
typedef int (*tm_storage_query_date_updated_after_f)(tm_storage *, time_t, tm_query **);
typedef int (*tm_storage_query_date_updated_before_f)(tm_storage *, time_t, tm_query **);
typedef int (*tm_storage_query_and_f)(tm_storage *, unsigned int, tm_query **, tm_query **);

struct tm_storage
{
    void *data;
    tm_storage_free_f free_f;
    tm_storage_get_tasks_f get_tasks_f;

    tm_storage_add_tasks_f add_tasks_f;
    tm_storage_modify_tasks_f modify_tasks_f;
    tm_storage_remove_tasks_f remove_tasks_f;

    tm_storage_get_task_f get_task_f;
    tm_storage_add_task_f add_task_f;
    tm_storage_modify_task_f modify_task_f;
    tm_storage_remove_task_f remove_task_f;

    tm_storage_get_task_diffs_f get_task_diffs_f;

    tm_storage_query_id_equals_f query_id_equals_f;
    tm_storage_query_name_contains_f query_name_contains_f;
    tm_storage_query_description_contains_f query_description_contains_f;
    tm_storage_query_status_contains_f query_status_contains_f;
    tm_storage_query_date_created_after_f query_date_created_after_f;
    tm_storage_query_date_created_before_f query_date_created_before_f;
    tm_storage_query_date_updated_after_f query_date_updated_after_f;
    tm_storage_query_date_updated_before_f query_date_updated_before_f;
    tm_storage_query_and_f query_and_f;
};

int tm_storage_free(tm_storage **);
int tm_storage_get_tasks(tm_storage *, tm_query *, tm_task ***, unsigned int *);

int tm_storage_add_tasks(tm_storage *, tm_task **tasks, unsigned int);
int tm_storage_modify_tasks(tm_storage *, tm_task **tasks, unsigned int);
int tm_storage_remove_tasks(tm_storage *, tm_task **tasks, unsigned int);

int tm_storage_get_task(tm_storage *, unsigned long, tm_task **);
int tm_storage_add_task(tm_storage *, tm_task *);
int tm_storage_modify_task(tm_storage *, tm_task *);
int tm_storage_remove_task(tm_storage *, tm_task *);

int tm_storage_get_task_diffs(tm_storage *, tm_task *, tm_task_diff ***, unsigned int *);

int tm_storage_query_id_equals(tm_storage *, unsigned long, tm_query **);
int tm_storage_query_name_contains(tm_storage *, char *, tm_query **);
int tm_storage_query_description_contains(tm_storage *, char *, tm_query **);
int tm_storage_query_status_contains(tm_storage *, char *, tm_query **);
int tm_storage_query_date_created_after(tm_storage *, time_t, tm_query **);
int tm_storage_query_date_created_before(tm_storage *, time_t, tm_query **);
int tm_storage_query_date_updated_after(tm_storage *, time_t, tm_query **);
int tm_storage_query_date_updated_before(tm_storage *, time_t, tm_query **);
int tm_storage_query_and(tm_storage *, unsigned int, tm_query **, tm_query **);
#endif
