/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "task.h"

int tm_task_new(tm_task **pthis)
{
    tm_task *this = 0;
    time_t now;

    if (0 == pthis || 0 != *pthis)
    {
        return 0;
    }

    now = time(0);
    this = (tm_task *)malloc(sizeof(struct tm_task));
    this->id = -1;

    this->name = 0;
    this->description = 0;
    this->status = strdup("new");
    this->date_created = now;
    this->date_updated = 0;

    this->did_id_change = 0;
    this->did_name_change = 0;
    this->did_description_change = 0;
    this->did_status_change = 0;
    *pthis = this;
    return 1;
}

int tm_task_free(tm_task **pthis)
{
    tm_task *this = 0;
    if (0 == pthis || 0 == *pthis)
    {
        return 0;
    }
    this = *pthis;
    if (0 != this->name)
    {
        free(this->name);
    }
    if (0 != this->description)
    {
        free(this->description);
    }
    if (0 != this->status)
    {
        free(this->status);
    }
    free(this);
    *pthis = 0;
    return 1;
}

int tm_task_arr_free(tm_task ***parr, unsigned int len)
{
    tm_task **arr = 0;
    unsigned int i;

    if (0 == parr || 0 == *parr)
    {
        return 0;
    }

    arr = *parr;
    for (i = 0; i < len; i++)
    {
        tm_task_free(&(arr[i]));
    }
    free(arr);
    *parr = 0;
    return 1;
}

int tm_task_arr_set_name(tm_task **arr, unsigned int len, char *name)
{
    unsigned int i;
    if (0 == arr || 0 == name)
    {
        return 0;
    }
    for (i = 0; i < len; i++)
    {
        tm_task_set_name(arr[i], name);
    }
    return 1;
}

int tm_task_arr_set_description(tm_task **arr, unsigned int len, char *description)
{
    unsigned int i;
    if (0 == arr || 0 == description)
    {
        return 0;
    }
    for (i = 0; i < len; i++)
    {
        tm_task_set_description(arr[i], description);
    }
    return 1;
}

int tm_task_arr_set_status(tm_task **arr, unsigned int len, char *status)
{
    unsigned int i;
    if (0 == arr || 0 == status)
    {
        return 0;
    }
    for (i = 0; i < len; i++)
    {
        tm_task_set_status(arr[i], status);
    }
    return 1;
}

int tm_task_get_id(tm_task *this, unsigned long *pid)
{
    if (0 == this || 0 == pid)
    {
        return 0;
    }
    *pid = this->id;
    return 1;
}

int tm_task_set_id(tm_task *this, unsigned long id)
{
    if (0 == this)
    {
        return 0;
    }
    this->id = id;
    this->did_id_change = 1;
    return 1;
}

int tm_task_get_date_created(tm_task *this, time_t *pdate)
{
    if (0 == this || 0 == pdate)
    {
        return 0;
    }
    *pdate = this->date_created;
    return 1;
}

int tm_task_set_date_created(tm_task *this, time_t date)
{
    if (0 == this)
    {
        return 0;
    }
    this->date_created = date;
    return 1;
}

int tm_task_get_date_updated(tm_task *this, time_t *pdate)
{
    if (0 == this || 0 == pdate)
    {
        return 0;
    }
    *pdate = this->date_updated;
    return 1;
}

int tm_task_set_date_updated(tm_task *this, time_t date)
{
    if (0 == this)
    {
        return 0;
    }
    this->date_updated = date;
    return 1;
}

int tm_task_get_name(tm_task *this, char **pname)
{
    if (0 == this || 0 == pname)
    {
        return 0;
    }
    *pname = this->name;
    return 1;
}

int tm_task_get_description(tm_task *this, char **pdescription)
{
    if (0 == this || 0 == pdescription)
    {
        return 0;
    }
    *pdescription = this->description;
    return 1;
}

int tm_task_get_status(tm_task *this, char **pstatus)
{
    if (0 == this || 0 == pstatus)
    {
        return 0;
    }
    *pstatus = this->status;
    return 1;
}

int tm_task_set_name(tm_task *this, char * name)
{
    if (0 == this || 0 == name)
    {
        return 0;
    }
    if (0 != this->name)
    {
        free(this->name);
    }
    this->name = strdup(name);
    this->did_name_change = 1;
    return 1;
}

int tm_task_set_description(tm_task *this, char * description)
{
    if (0 == this || 0 == description)
    {
        return 0;
    }
    if (0 != this->description)
    {
        free(this->description);
    }
    this->description = strdup(description);
    this->did_description_change = 1;
    return 1;
}

int tm_task_set_status(tm_task *this, char * status)
{
    if (0 == this || 0 == status)
    {
        return 0;
    }
    if (0 != this->status)
    {
        free(this->status);
    }
    this->status = strdup(status);
    this->did_status_change = 1;
    return 1;
}

int tm_task_invalidate(tm_task *this)
{
    this->did_id_change = 0;
    this->did_name_change = 0;
    this->did_description_change = 0;
    this->did_status_change = 0;
    return 1;
}
