/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "task_diff.h"

int tm_task_diff_new(tm_task_diff **pthis)
{
    tm_task_diff *this = 0;
    if (0 == pthis || 0 != *pthis)
    {
        return 0;
    }
    this = (tm_task_diff *)malloc(sizeof(struct tm_task_diff));
    this->id = -1;
    this->task = -1;

    this->name = 0;
    this->value = 0;
    this->date = 0;

    *pthis = this;
    return 1;
}

int tm_task_diff_free(tm_task_diff **pthis)
{
    tm_task_diff *this = 0;
    if (0 == pthis || 0 == *pthis)
    {
        return 0;
    }
    this = *pthis;
    if (0 != this->name)
    {
        free(this->name);
    }
    if (0 != this->value)
    {
        free(this->value);
    }
    free(this);
    *pthis = 0;
    return 1;
}

int tm_task_diff_get_id(tm_task_diff *this, unsigned long *pid)
{
    if (0 == this || 0 == pid)
    {
        return 0;
    }
    *pid = this->id;
    return 1;
}

int tm_task_diff_set_id(tm_task_diff *this, unsigned long id)
{
    if (0 == this)
    {
        return 0;
    }
    this->id = id;
    return 1;
}

int tm_task_diff_get_date(tm_task_diff *this, time_t *pdate)
{
    if (0 == this || 0 == pdate)
    {
        return 0;
    }
    *pdate = this->date;
    return 1;
}

int tm_task_diff_set_date(tm_task_diff *this, time_t date)
{
    if (0 == this)
    {
        return 0;
    }
    this->date = date;
    return 1;
}

int tm_task_diff_get_task(tm_task_diff *this, unsigned long *ptask)
{
    if (0 == this || 0 == ptask)
    {
        return 0;
    }
    *ptask = this->task;
    return 1;
}

int tm_task_diff_set_task(tm_task_diff *this, unsigned long task)
{
    if (0 == this)
    {
        return 0;
    }
    this->task = task;
    return 1;
}

int tm_task_diff_get_name(tm_task_diff *this, char **pname)
{
    if (0 == this || 0 == pname)
    {
        return 0;
    }
    *pname = this->name;
    return 1;
}

int tm_task_diff_get_value(tm_task_diff *this, char **pvalue)
{
    if (0 == this || 0 == pvalue)
    {
        return 0;
    }
    *pvalue = this->value;
    return 1;
}

int tm_task_diff_set_name(tm_task_diff *this, char * name)
{
    if (0 == this || 0 == name)
    {
        return 0;
    }
    if (0 != this->name)
    {
        free(this->name);
    }
    this->name = strdup(name);
    return 1;
}

int tm_task_diff_set_value(tm_task_diff *this, char * value)
{
    if (0 == this || 0 == value)
    {
        return 0;
    }
    if (0 != this->value)
    {
        free(this->value);
    }
    this->value = strdup(value);
    return 1;
}

int tm_task_diff_arr_free(tm_task_diff ***parr, unsigned int len)
{
    tm_task_diff **arr = 0;
    unsigned int i;

    if (0 == parr || 0 == *parr)
    {
        return 0;
    }

    arr = *parr;
    for (i = 0; i < len; i++)
    {
        tm_task_diff_free(&(arr[i]));
    }
    free(arr);
    *parr = 0;
    return 1;
}
