/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <stdlib.h>

#include "sqlite.h"
#include "tm.h"

int tm_new(tm_storage *storage, tm_lib **pthis)
{
    tm_lib *this = 0;
    if (0 == storage || 0 == pthis || 0 != *pthis)
    {
        return 0;
    }
    this = (tm_lib *)malloc(sizeof(struct tm_lib));
    this->storage = storage;
    *pthis = this;
    return 1;
}

int tm_free(tm_lib **pthis)
{
    tm_lib *this = 0;
    if (0 == pthis || 0 == *pthis)
    {
        return 0;
    }
    this = *pthis;
    tm_storage_free(&(this->storage));
    free(this);
    *pthis = 0;
    return 1;
}

int tm_add_tasks(tm_lib *this, tm_task **tasks, unsigned int len)
{
    if (0 == this || 0 == tasks)
    {
        return 0;
    }

    return tm_storage_add_tasks(this->storage, tasks, len);
}

int tm_get_tasks(tm_lib *this, tm_query *query, tm_task ***ptasks, unsigned int *plen)
{
    if (0 == this || 0 == ptasks || 0 == plen)
    {
        return 0;
    }

    return tm_storage_get_tasks(this->storage, query, ptasks, plen);
}

int tm_run_query(tm_lib *this, tm_query *query, tm_task ***ptasks, unsigned int *plen)
{
    return tm_get_tasks(this, query, ptasks, plen);
}

int tm_modify_tasks(tm_lib *this, tm_task **tasks, unsigned int len)
{
    if (0 == this || 0 == tasks)
    {
        return 0;
    }

    return tm_storage_modify_tasks(this->storage, tasks, len);
}

int tm_remove_tasks(tm_lib *this, tm_task **tasks, unsigned int len)
{
    if (0 == this || 0 == tasks)
    {
        return 0;
    }

    return tm_storage_remove_tasks(this->storage, tasks, len);
}

int tm_get_task(tm_lib *this, unsigned long id, tm_task **ptask)
{
    if (0 == this || 0 == ptask)
    {
        return 0;
    }

    return tm_storage_get_task(this->storage, id, ptask);
}

int tm_add_task(tm_lib *this, tm_task *task)
{
    if (0 == this || 0 == task)
    {
        return 0;
    }

    return tm_storage_add_task(this->storage, task);
}

int tm_modify_task(tm_lib *this, tm_task *task)
{
    if (0 == this || 0 == task)
    {
        return 0;
    }

    return tm_storage_modify_task(this->storage, task);
}

int tm_remove_task(tm_lib *this, tm_task *task)
{
    if (0 == this || 0 == task)
    {
        return 0;
    }

    return tm_storage_remove_task(this->storage, task);
}

int tm_get_task_diffs(tm_lib *this, tm_task *task, tm_task_diff ***parr, unsigned int *plen)
{
    if (0 == this || 0 == task || 0 == parr || 0 == plen)
    {
        return 0;
    }

    return tm_storage_get_task_diffs(this->storage, task, parr, plen);
}
