/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <criterion/criterion.h>

#include <tm.h>

TestSuite(usage);

Test(usage, tm)
{
    tm_lib *this = (tm_lib *)1;
    tm_storage *storage = (tm_storage *)1;

    cr_expect_eq(0, tm_new(0, 0));
    cr_expect_eq(0, tm_new(storage, &this));
}

Test(usage, storage_sqlite)
{
    tm_storage *storage = (tm_storage *)1;

    cr_expect_eq(0, tm_storage_sqlite(0, 0));
    cr_expect_eq(0, tm_storage_sqlite(0, &storage));
    cr_expect_eq(0, tm_storage_sqlite("", 0));
}

Test(usage, get_tasks)
{
    tm_lib *this = (tm_lib *)1;
    tm_query *query = (tm_query *)1;
    tm_task **tasks = 0;

    cr_expect_eq(0, tm_get_tasks(0, 0, 0, 0));
    cr_expect_eq(0, tm_get_tasks(this, 0, 0, 0));
    cr_expect_eq(0, tm_get_tasks(0, query, 0, 0));
    cr_expect_eq(0, tm_get_tasks(this, query, 0, 0));
    cr_expect_eq(0, tm_get_tasks(0, 0, &tasks, 0));
    cr_expect_eq(0, tm_get_tasks(this, 0, &tasks, 0));
    cr_expect_eq(0, tm_get_tasks(0, query, &tasks, 0));
}

Test(usage, get_task)
{
    tm_lib *this = (tm_lib *)1;
    tm_task *task = 0;

    cr_expect_eq(0, tm_get_task(0, 0, 0));
    cr_expect_eq(0, tm_get_task(this, 0, 0));
    cr_expect_eq(0, tm_get_task(0, 0, &task));
}

Test(usage, modify_task)
{
    tm_lib *this = (tm_lib *)1;
    tm_task *task = (tm_task *)1;

    cr_expect_eq(0, tm_modify_task(0, 0));
    cr_expect_eq(0, tm_modify_task(this, 0));
    cr_expect_eq(0, tm_modify_task(0, task));
}

Test(usage, remove_task)
{
    tm_lib *this = (tm_lib *)1;
    tm_task *task = (tm_task *)1;

    cr_expect_eq(0, tm_remove_task(0, 0));
    cr_expect_eq(0, tm_remove_task(this, 0));
    cr_expect_eq(0, tm_remove_task(0, task));
}

Test(usage, query_free)
{
    tm_query *query = 0;

    cr_expect_eq(0, tm_query_free(0));
    cr_expect_eq(0, tm_query_free(&query));
}

Test(usage, query_id_equals)
{
    tm_lib *this = (tm_lib *)1;
    tm_query *query = (tm_query *)1;

    cr_expect_eq(0, tm_query_id_equals(0, 0, 0));
    cr_expect_eq(0, tm_query_id_equals(this, 0, 0));
    cr_expect_eq(0, tm_query_id_equals(0, 0, &query));
}

Test(usage, query_date_created_after)
{
    tm_lib *this = (tm_lib *)1;
    tm_query *query = (tm_query *)1;

    cr_expect_eq(0, tm_query_date_created_after(0, 0, 0));
    cr_expect_eq(0, tm_query_date_created_after(this, 0, 0));
    cr_expect_eq(0, tm_query_date_created_after(0, 0, &query));
}

Test(usage, query_date_created_before)
{
    tm_lib *this = (tm_lib *)1;
    tm_query *query = (tm_query *)1;

    cr_expect_eq(0, tm_query_date_created_before(0, 0, 0));
    cr_expect_eq(0, tm_query_date_created_before(this, 0, 0));
    cr_expect_eq(0, tm_query_date_created_before(0, 0, &query));
}

Test(usage, query_date_updated_after)
{
    tm_lib *this = (tm_lib *)1;
    tm_query *query = (tm_query *)1;

    cr_expect_eq(0, tm_query_date_updated_after(0, 0, 0));
    cr_expect_eq(0, tm_query_date_updated_after(this, 0, 0));
    cr_expect_eq(0, tm_query_date_updated_after(0, 0, &query));
}

Test(usage, query_date_updated_before)
{
    tm_lib *this = (tm_lib *)1;
    tm_query *query = (tm_query *)1;

    cr_expect_eq(0, tm_query_date_updated_before(0, 0, 0));
    cr_expect_eq(0, tm_query_date_updated_before(this, 0, 0));
    cr_expect_eq(0, tm_query_date_updated_before(0, 0, &query));
}

Test(usage, query_name_contains)
{
    tm_lib *this = (tm_lib *)1;
    tm_query *query = (tm_query *)1;

    cr_expect_eq(0, tm_query_name_contains(0, 0, 0));
    cr_expect_eq(0, tm_query_name_contains(this, 0, 0));
    cr_expect_eq(0, tm_query_name_contains(0, "", 0));
    cr_expect_eq(0, tm_query_name_contains(this, "", 0));
    cr_expect_eq(0, tm_query_name_contains(0, 0, &query));
    cr_expect_eq(0, tm_query_name_contains(this, 0, &query));
    cr_expect_eq(0, tm_query_name_contains(0, "", &query));
}

Test(usage, query_description_contains)
{
    tm_lib *this = (tm_lib *)1;
    tm_query *query = (tm_query *)1;

    cr_expect_eq(0, tm_query_description_contains(0, 0, 0));
    cr_expect_eq(0, tm_query_description_contains(this, 0, 0));
    cr_expect_eq(0, tm_query_description_contains(0, "", 0));
    cr_expect_eq(0, tm_query_description_contains(this, "", 0));
    cr_expect_eq(0, tm_query_description_contains(0, 0, &query));
    cr_expect_eq(0, tm_query_description_contains(this, 0, &query));
    cr_expect_eq(0, tm_query_description_contains(0, "", &query));
}

Test(usage, query_status_contains)
{
    tm_lib *this = (tm_lib *)1;
    tm_query *query = (tm_query *)1;

    cr_expect_eq(0, tm_query_status_contains(0, 0, 0));
    cr_expect_eq(0, tm_query_status_contains(this, 0, 0));
    cr_expect_eq(0, tm_query_status_contains(0, "", 0));
    cr_expect_eq(0, tm_query_status_contains(this, "", 0));
    cr_expect_eq(0, tm_query_status_contains(0, 0, &query));
    cr_expect_eq(0, tm_query_status_contains(this, 0, &query));
    cr_expect_eq(0, tm_query_status_contains(0, "", &query));
}

Test(usage, query_and)
{
    tm_lib *this = (tm_lib *)1;
    tm_query *query = (tm_query *)1;
    tm_query **queries = (tm_query **)1;

    cr_expect_eq(0, tm_query_and(0, 0, 0, 0));
    cr_expect_eq(0, tm_query_and(this, 0, 0, 0));
    cr_expect_eq(0, tm_query_and(0, 0, queries, 0));
    cr_expect_eq(0, tm_query_and(this, 0, queries, 0));
    cr_expect_eq(0, tm_query_and(0, 0, 0, &query));
    cr_expect_eq(0, tm_query_and(this, 0, 0, &query));
    cr_expect_eq(0, tm_query_and(0, 0, queries, &query));
}

Test(usage, task_new)
{
    tm_task *task = (tm_task *)1;

    cr_expect_eq(0, tm_task_new(0));
    cr_expect_eq(0, tm_task_new(&task));
}

Test(usage, task_get_id)
{
    cr_expect_eq(0, tm_task_get_id(0, 0));
}

Test(usage, task_set_id)
{
    cr_expect_eq(0, tm_task_set_id(0, 0));
}

Test(usage, task_get_date_created)
{
    cr_expect_eq(0, tm_task_get_date_created(0, 0));
}

Test(usage, task_get_date_updated)
{
    cr_expect_eq(0, tm_task_get_date_updated(0, 0));
}

Test(usage, task_get_name)
{
    tm_task *task = (tm_task *)1;
    char *name = 0;

    cr_expect_eq(0, tm_task_get_name(0, 0));
    cr_expect_eq(0, tm_task_get_name(task, 0));
    cr_expect_eq(0, tm_task_get_name(0, &name));
}

Test(usage, task_set_name)
{
    tm_task *task = (tm_task *)1;

    cr_expect_eq(0, tm_task_set_name(0, 0));
    cr_expect_eq(0, tm_task_set_name(task, 0));
    cr_expect_eq(0, tm_task_set_name(0, ""));
}

Test(usage, task_get_description)
{
    tm_task *task = (tm_task *)1;
    char *description = 0;

    cr_expect_eq(0, tm_task_get_description(0, 0));
    cr_expect_eq(0, tm_task_get_description(task, 0));
    cr_expect_eq(0, tm_task_get_description(0, &description));
}

Test(usage, task_get_status)
{
    tm_task *task = (tm_task *)1;
    char *status = 0;

    cr_expect_eq(0, tm_task_get_status(0, 0));
    cr_expect_eq(0, tm_task_get_status(task, 0));
    cr_expect_eq(0, tm_task_get_status(0, &status));
}

Test(usage, task_set_description)
{
    tm_task *task = (tm_task *)1;

    cr_expect_eq(0, tm_task_set_description(0, 0));
    cr_expect_eq(0, tm_task_set_description(task, 0));
    cr_expect_eq(0, tm_task_set_description(0, ""));
}

Test(usage, task_set_status)
{
    tm_task *task = (tm_task *)1;

    cr_expect_eq(0, tm_task_set_status(0, 0));
    cr_expect_eq(0, tm_task_set_status(task, 0));
    cr_expect_eq(0, tm_task_set_status(0, ""));
}

Test(usage, task_free)
{
    tm_task *task = 0;

    cr_expect_eq(0, tm_task_free(0));
    cr_expect_eq(0, tm_task_free(&task));
}

Test(usage, task_arr_free)
{
    cr_expect_eq(0, tm_task_arr_free(0, 0));
}

Test(usage, task_arr_set_name)
{
    tm_task **tasks = (tm_task **)1;
    cr_expect_eq(0, tm_task_arr_set_name(0, 0, 0));
    cr_expect_eq(0, tm_task_arr_set_name(tasks, 0, 0));
    cr_expect_eq(0, tm_task_arr_set_name(0, 0, ""));
}

Test(usage, task_arr_set_description)
{
    tm_task **tasks = (tm_task **)1;
    cr_expect_eq(0, tm_task_arr_set_description(0, 0, 0));
    cr_expect_eq(0, tm_task_arr_set_description(tasks, 0, 0));
    cr_expect_eq(0, tm_task_arr_set_description(0, 0, ""));
}

Test(usage, task_arr_set_status)
{
    tm_task **tasks = (tm_task **)1;
    cr_expect_eq(0, tm_task_arr_set_status(0, 0, 0));
    cr_expect_eq(0, tm_task_arr_set_status(tasks, 0, 0));
    cr_expect_eq(0, tm_task_arr_set_status(0, 0, ""));
}

Test(usage, add_tasks)
{
    tm_lib *this = (tm_lib *)1;
    tm_task **tasks = (tm_task **)1;

    cr_expect_eq(0, tm_add_tasks(0, 0, 0));
    cr_expect_eq(0, tm_add_tasks(0, tasks, 0));
    cr_expect_eq(0, tm_add_tasks(this, 0, 0));
}

Test(usage, modify_tasks)
{
    tm_lib *this = (tm_lib *)1;
    tm_task **tasks = (tm_task **)1;

    cr_expect_eq(0, tm_modify_tasks(0, 0, 0));
    cr_expect_eq(0, tm_modify_tasks(0, tasks, 0));
    cr_expect_eq(0, tm_modify_tasks(this, 0, 0));
}

Test(usage, remove_tasks)
{
    tm_lib *this = (tm_lib *)1;
    tm_task **tasks = (tm_task **)1;

    cr_expect_eq(0, tm_remove_tasks(0, 0, 0));
    cr_expect_eq(0, tm_remove_tasks(0, tasks, 0));
    cr_expect_eq(0, tm_remove_tasks(this, 0, 0));
}

Test(usage, tm_free)
{
    tm_lib *this = 0;

    cr_expect_eq(0, tm_free(0));
    cr_expect_eq(0, tm_free(&this));
}

Test(usage, task_diff_new)
{
    tm_task_diff *diff = (tm_task_diff *)1;

    cr_expect_eq(0, tm_task_diff_new(0));
    cr_expect_eq(0, tm_task_diff_new(&diff));
}

Test(usage, task_diff_get_id)
{
    cr_expect_eq(0, tm_task_diff_get_id(0, 0));
}

Test(usage, task_diff_set_id)
{
    cr_expect_eq(0, tm_task_diff_set_id(0, 0));
}

Test(usage, task_diff_get_date)
{
    cr_expect_eq(0, tm_task_diff_get_date(0, 0));
}

Test(usage, task_diff_get_task)
{
    cr_expect_eq(0, tm_task_diff_get_task(0, 0));
}

Test(usage, task_diff_set_task)
{
    cr_expect_eq(0, tm_task_diff_set_task(0, 0));
}

Test(usage, task_diff_get_name)
{
    tm_task_diff *task = (tm_task_diff *)1;
    char *name = 0;

    cr_expect_eq(0, tm_task_diff_get_name(0, 0));
    cr_expect_eq(0, tm_task_diff_get_name(task, 0));
    cr_expect_eq(0, tm_task_diff_get_name(0, &name));
}

Test(usage, task_diff_set_name)
{
    tm_task_diff *task = (tm_task_diff *)1;

    cr_expect_eq(0, tm_task_diff_set_name(0, 0));
    cr_expect_eq(0, tm_task_diff_set_name(task, 0));
    cr_expect_eq(0, tm_task_diff_set_name(0, ""));
}

Test(usage, task_diff_get_value)
{
    tm_task_diff *task = (tm_task_diff *)1;
    char *value = 0;

    cr_expect_eq(0, tm_task_diff_get_value(0, 0));
    cr_expect_eq(0, tm_task_diff_get_value(task, 0));
    cr_expect_eq(0, tm_task_diff_get_value(0, &value));
}

Test(usage, task_diff_set_value)
{
    tm_task_diff *task = (tm_task_diff *)1;

    cr_expect_eq(0, tm_task_diff_set_value(0, 0));
    cr_expect_eq(0, tm_task_diff_set_value(task, 0));
    cr_expect_eq(0, tm_task_diff_set_value(0, ""));
}

Test(usage, task_diff_free)
{
    tm_task_diff *diff = 0;

    cr_expect_eq(0, tm_task_diff_free(0));
    cr_expect_eq(0, tm_task_diff_free(&diff));
}
