'use strict';

const {getAddNewTask, getEditTask, getIndex, getNavBar, getTask,
  withTmsAndPuppeteer} = require('./tools');

const itsWithTmsAndPuppeteer = (closure) => {
  it('without js', async () => {
    await withTmsAndPuppeteer(closure(false));
  });
  it('with js', async () => {
    await withTmsAndPuppeteer(closure(true));
  });
};

describe('when on index page', () => {
  describe('it is visible', () => {
    const doIt = (js) => {
      return async (ip, port, page) => {
        await page.setJavaScriptEnabled(js);
        await page.goto('http://' + ip + ':' + port + '/');

        const index = await getIndex(page);
        await index.expectToBeVisible();
      };
    };

    itsWithTmsAndPuppeteer(doIt);
  });

  describe('adds', () => {
    const addOneTask = async (page, js, name, desc, status) => {
      const navbar = await getNavBar(page);
      await navbar.clickAddNewTask();
      if (!js) {
        await page.waitForNavigation();
      }

      const addNewTask = await getAddNewTask(page);
      await addNewTask.expectToBeVisible();

      await addNewTask.setName(name);
      await addNewTask.setDescription(desc);
      await addNewTask.setStatus(status);
      await addNewTask.clickSubmit();
      await page.waitForNavigation();
    };

    const expectTaskPage = async (page, id, name, desc, status) => {
      const task = await getTask(page);
      await task.expectToBeVisible();

      await task.expectNameToEqual(id, name);
      await task.expectDescriptionToEqual(desc);
      await task.expectStatusToEqual(status);
      await task.expectDateCreatedNotToBeEmpty();
      await task.expectDateUpdatedNotToBeEmpty();
      return task;
    };

    const expectOnlyTaskOnIndexResults = async (page, js, id, name, desc, status) => {
      const navbar = await getNavBar(page);
      await navbar.clickIndex();
      if (!js) {
        await page.waitForNavigation();
      }

      const index = await getIndex(page);
      await index.expectToBeVisible();
      await index.expectResultsToEqual([
        ['#' + id, name, status],
      ]);

      await index.clickResult(0);
      if (!js) {
        await page.waitForNavigation();
      }

      await expectTaskPage(page, id, name, desc, status);
    };

    describe('one task', () => {
      describe('and navigates to its page', () => {
        const doIt = (js) => {
          return async (ip, port, page) => {
            await page.setJavaScriptEnabled(js);
            await page.goto('http://' + ip + ':' + port + '/');

            await addOneTask(page, js, 'Buy milk', 'Goat milk', 'new');
            await expectTaskPage(page, '1', 'Buy milk', 'Goat milk', 'new');
          };
        };

        itsWithTmsAndPuppeteer(doIt);
      });

      describe('and clicks the link on index\' results', () => {
        const doIt = (js) => {
          return async (ip, port, page) => {
            await page.setJavaScriptEnabled(js);
            await page.goto('http://' + ip + ':' + port + '/');

            await addOneTask(page, js, 'Buy milk', 'Goat milk', 'new');
            await expectOnlyTaskOnIndexResults(page, js, '1', 'Buy milk', 'Goat milk', 'new');
          };
        };

        itsWithTmsAndPuppeteer(doIt);
      });

      describe('then edits it', () => {
        const editTheTask = async (page, id, name, desc, status) => {
          const task = await getTask(page);
          await task.clickEdit();
          await page.waitForNavigation();

          const edit = await getEditTask(page);
          await edit.expectToBeVisible();

          await edit.setName(name);
          await edit.setDescription(desc);
          await edit.setStatus(status);
          await edit.clickSubmit();
          await page.waitForNavigation();
        };

        describe('and navigates to its page', () => {
          const doIt = (js) => {
            return async (ip, port, page) => {
              await page.setJavaScriptEnabled(js);
              await page.goto('http://' + ip + ':' + port + '/');

              await addOneTask(page, js, 'Buy milk', 'Goat milk', 'new');
              await editTheTask(page, '1', 'Buy beer', 'Lager', 'open');
              await expectTaskPage(page, '1', 'Buy beer', 'Lager', 'open');
            };
          };

          itsWithTmsAndPuppeteer(doIt);
        });

        describe('and clicks the link on index\' results', () => {
          const doIt = (js) => {
            return async (ip, port, page) => {
              await page.setJavaScriptEnabled(js);
              await page.goto('http://' + ip + ':' + port + '/');

              await addOneTask(page, js, 'Buy milk', 'Goat milk', 'new');
              await editTheTask(page, '1', 'Buy beer', 'Lager', 'open');
              await expectOnlyTaskOnIndexResults(page, js, '1', 'Buy beer', 'Lager', 'open');
            };
          };

          itsWithTmsAndPuppeteer(doIt);
        });
      });
    });

    describe('many tasks and queries one', () => {
      const doIt = (js) => {
        return async (ip, port, page) => {
          await page.setJavaScriptEnabled(js);
          await page.goto('http://' + ip + ':' + port + '/');

          await addOneTask(page, js, 'Buy milk', 'Goat milk', 'new');
          await addOneTask(page, js, 'Buy milk', 'Goat milk', 'new');
          await addOneTask(page, js, 'Buy milk', 'Goat milk', 'new');
          await addOneTask(page, js, 'Buy milk', 'Goat milk', 'new');
          await addOneTask(page, js, 'Buy milk', 'Goat milk', 'new');

          const navbar = await getNavBar(page);
          await navbar.clickIndex();
          if (!js) {
            await page.waitForNavigation();
          }

          let index = await getIndex(page);
          await index.expectToBeVisible();

          await index.setQueryIdEquals('3');
          await index.clickQuerySubmit();
          await page.waitForNavigation();

          index = await getIndex(page);
          await index.expectToBeVisible();
          await index.expectResultsToEqual([
            ['#3', 'Buy milk', 'new'],
          ]);
        };
      };

      itsWithTmsAndPuppeteer(doIt);
    });
  });
});
