/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <errno.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include <criterion/criterion.h>
#include <sqlite3.h>

#include "common.h"

TestSuite(date, .init = cache_timezone, .fini = restore_timezone);

static void update_creation_date_of_all_tasks(time_t date)
{
    char *query = "UPDATE task SET date_created = ?";
    sqlite3_stmt *stmt = 0;
    sqlite3 *db = 0;

    cr_log_info("update_creation_date_of_all_tasks date=%ld", date);

    cr_expect_eq(0, sqlite3_open("tm.db", &db));
    cr_expect_neq(0, db);

    sqlite3_prepare_v2(db, query, strlen(query), &stmt, 0);
    sqlite3_bind_int64(stmt, 1, date);

    cr_expect_eq(SQLITE_DONE, sqlite3_step(stmt));
    sqlite3_finalize(stmt);

    sqlite3_close(db);
}

static char *expected_output_when_found = "ID | Name\n"
    "1  | Buy milk\n";
static char *expected_output_when_not_found = "ID | Name\n";

static void prepare_db(void)
{
    struct tm nowtm;
    memset(&nowtm, 0, sizeof(struct tm));
    nowtm.tm_year = 70;
    nowtm.tm_mon = 0;
    nowtm.tm_mday = 0;
    nowtm.tm_hour = 0;
    nowtm.tm_min = 0;
    nowtm.tm_sec = 0;

    init_folder();
    do_add("Buy milk", 0);
    update_creation_date_of_all_tasks(mktime(&nowtm));
}

Test(date, dcgt)
{
    prepare_db();
    do_query(expected_output_when_found, 2, "dcgt", "1");
    clear_folder();
}

Test(date, dcgt_not_found)
{
    prepare_db();
    do_query(expected_output_when_not_found, 2, "dcgt", "2000");
    clear_folder();
}

Test(date, dclt)
{
    prepare_db();
    do_query(expected_output_when_found, 2, "dclt", "2000");
    clear_folder();
}

Test(date, dclt_not_found)
{
    prepare_db();
    do_query(expected_output_when_not_found, 2, "dclt", "1");
    clear_folder();
}

Test(date, invalid)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "dcgt";
    argv[3] = "x";
    argv[4] = 0;

    expect_usage(argv, 1);
}
