/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <errno.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include <criterion/criterion.h>

#include "common.h"

TestSuite(modify, .init = cache_timezone, .fini = restore_timezone);

Test(modify, filter_name_short)
{
    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "n", "Buy milk",
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_name_long)
{
    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "name", "Buy milk",
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_description_short)
{
    init_folder();
    do_add("Buy milk", 2, "d", "Goat milk");
    do_modify(1, 2, 2,
        "d", "Goat milk",
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_description_long)
{
    init_folder();
    do_add("Buy milk", 2, "d", "Goat milk");
    do_modify(1, 2, 2,
        "description", "Goat milk",
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_status_short)
{
    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "s", "new",
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_status_long)
{
    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "status", "new",
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_created_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "dc", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_created_after_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "dcgt", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_created_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "date-created", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_created_after_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "date-created-after", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_created_before_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "dclt", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_created_before_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "date-created-before", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_updated_after_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "dugt", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_updated_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "du", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_updated_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "date-updated", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_updated_after_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "date-updated-after", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_updated_before_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "dult", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, filter_date_updated_before_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];
    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "date-updated-before", tmp,
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    clear_folder();
}

Test(modify, modifier_name_short)
{
    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "i", "1",
        "n", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    do_print(1, 1, 5,
        "Name", "Buy beer",
        "Status", "new",
        "Date created", LAST_1_MINUTE,
        "Date updated", LAST_1_MINUTE,
        "History", 3,
            LAST_1_MINUTE, "changed", "name", "Buy milk", "Buy beer",
            LAST_1_MINUTE, "added", "status", "new",
            LAST_1_MINUTE, "added", "name", "Buy milk");
    clear_folder();
}

Test(modify, modifier_name_long)
{
    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "i", "1",
        "name", "Buy beer");
    do_query("ID | Name\n1  | Buy beer\n", 0);
    do_print(1, 1, 5,
        "Name", "Buy beer",
        "Status", "new",
        "Date created", LAST_1_MINUTE,
        "Date updated", LAST_1_MINUTE,
        "History", 3,
            LAST_1_MINUTE, "changed", "name", "Buy milk", "Buy beer",
            LAST_1_MINUTE, "added", "status", "new",
            LAST_1_MINUTE, "added", "name", "Buy milk");
    clear_folder();
}

Test(modify, modifier_description_short)
{
    init_folder();
    do_add("Buy milk", 2, "d", "Goat milk");
    do_modify(1, 2, 2,
        "i", "1",
        "d", "Yak milk");
    do_query("ID | Name\n"
        "1  | Buy milk\n", 0);
    do_print(1, 1, 6,
        "Name", "Buy milk",
        "Description", "Yak milk",
        "Status", "new",
        "Date created", LAST_1_MINUTE,
        "Date updated", LAST_1_MINUTE,
        "History", 4,
            LAST_1_MINUTE, "changed", "description", "Goat milk", "Yak milk",
            LAST_1_MINUTE, "added", "status", "new",
            LAST_1_MINUTE, "added", "description", "Goat milk",
            LAST_1_MINUTE, "added", "name", "Buy milk");
    clear_folder();
}

Test(modify, modifier_description_short_removed)
{
    init_folder();
    do_add("Buy milk", 2, "d", "Goat milk");
    do_modify(1, 2, 2,
        "i", "1",
        "d", "");
    do_query("ID | Name\n"
        "1  | Buy milk\n", 0);
    do_print(1, 1, 5,
        "Name", "Buy milk",
        "Status", "new",
        "Date created", LAST_1_MINUTE,
        "Date updated", LAST_1_MINUTE,
        "History", 4,
            LAST_1_MINUTE, "removed", "description", "Goat milk",
            LAST_1_MINUTE, "added", "status", "new",
            LAST_1_MINUTE, "added", "description", "Goat milk",
            LAST_1_MINUTE, "added", "name", "Buy milk");
    clear_folder();
}

Test(modify, modifier_description_long)
{
    init_folder();
    do_add("Buy milk", 2, "d", "Goat milk");
    do_modify(1, 2, 2,
        "i", "1",
        "description", "Yak milk");
    do_query("ID | Name\n"
        "1  | Buy milk\n", 0);
    do_print(1, 1, 6,
        "Name", "Buy milk",
        "Description", "Yak milk",
        "Status", "new",
        "Date created", LAST_1_MINUTE,
        "Date updated", LAST_1_MINUTE,
        "History", 4,
            LAST_1_MINUTE, "changed", "description", "Goat milk", "Yak milk",
            LAST_1_MINUTE, "added", "status", "new",
            LAST_1_MINUTE, "added", "description", "Goat milk",
            LAST_1_MINUTE, "added", "name", "Buy milk");
    clear_folder();
}

Test(modify, modifier_description_long_removed)
{
    init_folder();
    do_add("Buy milk", 2, "description", "Goat milk");
    do_modify(1, 2, 2,
        "i", "1",
        "description", "");
    do_query("ID | Name\n"
        "1  | Buy milk\n", 0);
    do_print(1, 1, 5,
        "Name", "Buy milk",
        "Status", "new",
        "Date created", LAST_1_MINUTE,
        "Date updated", LAST_1_MINUTE,
        "History", 4,
            LAST_1_MINUTE, "removed", "description", "Goat milk",
            LAST_1_MINUTE, "added", "status", "new",
            LAST_1_MINUTE, "added", "description", "Goat milk",
            LAST_1_MINUTE, "added", "name", "Buy milk");
    clear_folder();
}

Test(modify, modifier_status_short)
{
    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "i", "1",
        "s", "open");
    do_query("ID | Name\n"
        "1  | Buy milk\n", 0);
    do_print(1, 1, 5,
        "Name", "Buy milk",
        "Status", "open",
        "Date created", LAST_1_MINUTE,
        "Date updated", LAST_1_MINUTE,
        "History", 3,
            LAST_1_MINUTE, "changed", "status", "new", "open",
            LAST_1_MINUTE, "added", "status", "new",
            LAST_1_MINUTE, "added", "name", "Buy milk");
    clear_folder();
}

Test(modify, modifier_status_long)
{
    init_folder();
    do_add("Buy milk", 0);
    do_modify(1, 2, 2,
        "i", "1",
        "status", "open");
    do_query("ID | Name\n"
        "1  | Buy milk\n", 0);
    do_print(1, 1, 5,
        "Name", "Buy milk",
        "Status", "open",
        "Date created", LAST_1_MINUTE,
        "Date updated", LAST_1_MINUTE,
        "History", 3,
            LAST_1_MINUTE, "changed", "status", "new", "open",
            LAST_1_MINUTE, "added", "status", "new",
            LAST_1_MINUTE, "added", "name", "Buy milk");
    clear_folder();
}
