/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <errno.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include <criterion/criterion.h>

#include "common.h"

TestSuite(remove, .init = cache_timezone, .fini = restore_timezone);

Test(remove, filter_id_short)
{
    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "i", "1");
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "i", "1");
    clear_folder();
}

Test(remove, filter_id_long)
{
    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "id", "1");
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "id", "1");
    clear_folder();
}

Test(remove, filter_name_short)
{
    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "n", "milk");
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "n", "milk");
    clear_folder();
}

Test(remove, filter_name_long)
{
    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "name", "milk");
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "name", "milk");
    clear_folder();
}

Test(remove, filter_description_short)
{
    init_folder();
    do_add("Buy milk", 2, "d", "Goat milk");
    do_remove(1, 2, "d", "Goat");
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "d", "Goat");
    clear_folder();
}

Test(remove, filter_description_long)
{
    init_folder();
    do_add("Buy milk", 2, "d", "Goat milk");
    do_remove(1, 2, "description", "Goat");
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "description", "Goat");
    clear_folder();
}

Test(remove, filter_status_short)
{
    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "s", "new");
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "s", "new");
    clear_folder();
}

Test(remove, filter_status_long)
{
    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "status", "new");
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "status", "new");
    clear_folder();
}

Test(remove, filter_date_created_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "dc", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "dc", tmp);
    clear_folder();
}

Test(remove, filter_date_created_after_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "dcgt", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "dcgt", tmp);
    clear_folder();
}

Test(remove, filter_date_created_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "date-created", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "date-created", tmp);
    clear_folder();
}

Test(remove, filter_date_created_after_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "date-created-after", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "date-created-after", tmp);
    clear_folder();
}

Test(remove, filter_date_created_before_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "dclt", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "dclt", tmp);
    clear_folder();
}

Test(remove, filter_date_created_before_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "date-created-before", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "date-created-before", tmp);
    clear_folder();
}

Test(remove, filter_date_updated_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "du", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "du", tmp);
    clear_folder();
}

Test(remove, filter_date_updated_after_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "dugt", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "dugt", tmp);
    clear_folder();
}

Test(remove, filter_date_updated_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "date-updated", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "date-updated", tmp);
    clear_folder();
}

Test(remove, filter_date_updated_after_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "date-updated-after", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "date-updated-after", tmp);
    clear_folder();
}

Test(remove, filter_date_updated_before_short)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "dult", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "dult", tmp);
    clear_folder();
}

Test(remove, filter_date_updated_before_long)
{
    time_t now = time(0);
    struct tm nowtm;
    char tmp[64];

    localtime_r(&now, &nowtm);
    sprintf(tmp, "%u", nowtm.tm_year + 1900);

    init_folder();
    do_add("Buy milk", 0);
    do_remove(1, 2, "date-updated-before", tmp);
    do_query("ID | Name\n", 0);
    do_print(0, 1, 0);
    do_remove(0, 2, "date-updated-before", tmp);
    clear_folder();
}
