/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <criterion/criterion.h>

#include "common.h"

TestSuite(usage, .init = cache_timezone, .fini = restore_timezone);

Test(usage, no_args)
{
    char *argv[2];
    argv[0] = "./tm";
    argv[1] = 0;

    expect_usage(argv, 1);
}

Test(usage, unknown_command)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "kra";
    argv[2] = 0;

    expect_usage(argv, 1);
}

Test(usage, help_long)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "--help";
    argv[2] = 0;

    expect_usage(argv, 0);
}

Test(usage, help_short)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "-h";
    argv[2] = 0;

    expect_usage(argv, 0);
}

Test(usage, unknown_opt)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "-z";
    argv[2] = 0;

    expect_usage(argv, 1);
}

Test(usage, dbfile_short_no_arg)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "-d";
    argv[2] = 0;

    expect_usage(argv, 1);
}

Test(usage, dbfile_long_no_arg)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "--dbfile";
    argv[2] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_short_no_name)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_short_modifier_unknown)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "unknown";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_short_modifier_name_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "n";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_short_modifier_name_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "name";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_short_modifier_name_short_value)
{
    char *argv[6];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "n";
    argv[4] = "Buy milk";
    argv[5] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_short_modifier_name_long_value)
{
    char *argv[6];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "name";
    argv[4] = "Buy milk";
    argv[5] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_short_modifier_description_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "d";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_short_modifier_description_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "description";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_short_modifier_status_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "s";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_short_modifier_status_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "status";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_long_no_name)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_long_modifier_name_unknown)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "unknown";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_long_modifier_name_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "n";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_long_modifier_name_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "name";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_long_modifier_name_short_value)
{
    char *argv[6];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "n";
    argv[4] = "Buy milk";
    argv[5] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_long_modifier_name_long_value)
{
    char *argv[6];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "name";
    argv[4] = "Buy milk";
    argv[5] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_long_modifier_description_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "d";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_long_modifier_description_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "description";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_long_modifier_status_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "s";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, add_long_modifier_status_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = "Buy milk";
    argv[3] = "status";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_unknown)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "unknown";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_id_equals_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "i";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_id_equals_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "id";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_name_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "n";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_name_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "name";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_description_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "d";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_description_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "description";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_status_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "s";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_status_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "status";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_created_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "dc";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_created_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "date-created";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_created_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "dcgt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_created_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "date-created-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_created_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "dclt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_created_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "date-created-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_updated_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "du";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_updated_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "date-updated";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_updated_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "dugt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_updated_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "date-updated-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_updated_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "dult";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_filter_date_updated_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "date-updated-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_no_modifier)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "--";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_modifier_unknown)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "--";
    argv[3] = "unknown";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_modifier_name_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "--";
    argv[3] = "n";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_modifier_name_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "--";
    argv[3] = "name";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_modifier_description_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "--";
    argv[3] = "d";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_modifier_description_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "n";
    argv[2] = "--";
    argv[3] = "description";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_modifier_status_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "--";
    argv[3] = "s";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_short_modifier_status_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "n";
    argv[2] = "--";
    argv[3] = "status";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_unknown)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "unknown";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_id_equals_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "i";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_id_equals_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "id";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_name_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "n";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_name_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "name";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_description_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "d";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_description_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "description";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_status_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "s";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_status_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "status";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_created_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "dc";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_created_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "date-created";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_created_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "dcgt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_created_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "date-created-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_created_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "dclt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_created_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "date-created-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_updated_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "du";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_updated_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "date-updated";
    argv[3] = 0;

    expect_usage(argv, 1);
}
Test(usage, modify_long_filter_date_updated_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "dugt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_updated_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "date-updated-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_updated_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "dult";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_filter_date_updated_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "date-updated-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_no_modifier)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "--";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_modifier_name_unknown)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "--";
    argv[3] = "unknown";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_modifier_name_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "--";
    argv[3] = "n";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_modifier_name_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "--";
    argv[3] = "name";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_modifier_description_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "--";
    argv[3] = "d";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_modifier_description_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "n";
    argv[2] = "--";
    argv[3] = "description";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_modifier_status_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "--";
    argv[3] = "s";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, modify_long_modifier_status_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "n";
    argv[2] = "--";
    argv[3] = "status";
    argv[4] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_unknown)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "unknown";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_id_equals_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "i";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_id_equals_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "id";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_name_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "n";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_name_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "name";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_description_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "d";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_description_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "description";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_status_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "s";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_status_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "status";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_created_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "dc";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_created_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "date-created";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_created_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "dcgt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_created_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "date-created-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_created_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "dclt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_created_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "date-created-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_updated_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "du";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_updated_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "date-updated";
    argv[3] = 0;

    expect_usage(argv, 1);
}
Test(usage, query_short_filter_date_updated_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "dugt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_updated_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "date-updated-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_updated_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "dult";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_short_filter_date_updated_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "date-updated-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_unknown)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "unknown";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_id_equals_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "i";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_id_equals_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "id";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_name_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "n";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_name_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "name";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_description_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "d";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_description_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "description";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_status_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "s";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_status_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "status";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_date_created_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "dc";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_date_created_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "date-created";
    argv[3] = 0;

    expect_usage(argv, 1);
}
Test(usage, query_long_filter_date_created_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "dcgt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_date_created_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "date-created-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_date_created_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "dclt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_date_created_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "date-created-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_date_updated_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "du";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_date_updated_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "date-updated";
    argv[3] = 0;

    expect_usage(argv, 1);
}
Test(usage, query_long_filter_date_updated_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "dugt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_date_updated_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "date-updated-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_date_updated_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "dult";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, query_long_filter_date_updated_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "date-updated-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_unknown)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "unknown";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_id_equals_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "i";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_id_equals_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "id";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_name_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "n";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_name_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "name";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_description_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "d";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_description_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "description";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_status_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "s";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_status_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "status";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_created_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "dc";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_created_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "date-created";
    argv[3] = 0;

    expect_usage(argv, 1);
}


Test(usage, remove_short_filter_date_created_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "dcgt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_created_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "date-created-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_created_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "dclt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_created_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "date-created-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_updated_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "du";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_updated_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "date-updated";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_updated_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "dugt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_updated_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "date-updated-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_updated_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "dult";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_short_filter_date_updated_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "r";
    argv[2] = "date-updated-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_unknown)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "unknown";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_id_equals_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "i";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_id_equals_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "id";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_name_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "n";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_name_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "name";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_description_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "d";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_description_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "description";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_status_contains_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "s";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_status_contains_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "status";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_created_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "dc";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_created_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "date-created";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_created_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "dcgt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_created_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "date-created-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_created_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "dclt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_created_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "date-created-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_updated_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "du";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_updated_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "date-updated";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_updated_after_short_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "dugt";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_updated_after_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "date-updated-after";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_updated_before_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "dult";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, remove_long_filter_date_updated_before_long_no_value)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "remove";
    argv[2] = "date-updated-before";
    argv[3] = 0;

    expect_usage(argv, 1);
}

Test(usage, print_short_no_id)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "p";
    argv[2] = 0;

    expect_usage(argv, 1);
}

Test(usage, print_long_no_id)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "print";
    argv[2] = 0;

    expect_usage(argv, 1);
}
