/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <stdio.h>
#include <string.h>
#include <signal.h>

#include <static_index_html.h>
#include <tms.h>

typedef struct add_html_userdata add_html_userdata;
typedef struct diff_text diff_text;

enum add_html_entered
{
    THE_NULL,
    THE_ADD
};

struct add_html_userdata
{
    enum add_html_entered entered;

    int add_i;
};

static int tag_top(char **pout, unsigned int *plen, void *userdata, char *tag)
{
    if (0 == strcmp("edit-style", tag) ||
        0 == strcmp("error-style", tag) ||
        0 == strcmp("index-style", tag) ||
        0 == strcmp("task-style", tag) ||
        0 == strcmp("warning-style", tag))
    {
        char *tmp = "style='display: none;'";
        return tpl_append(pout, plen, tmp, strlen(tmp));
    }
    else if (0 == strcmp("add-style", tag))
    {
        return 1;
    }
    return 0;
}

static int tag(char **pout, unsigned int *plen, void *userdata, char *tag)
{
    add_html_userdata *puserdata = (add_html_userdata *)userdata;
    switch (puserdata->entered)
    {
        case THE_NULL:
            return tag_top(pout, plen, userdata, tag);
        default:
            break;
    }
    return 0;
}

static int enter(char **pout, unsigned int *plen, void *userdata,
    char *name)
{
    add_html_userdata *puserdata = (add_html_userdata *)userdata;
    switch (puserdata->entered)
    {
        case THE_NULL:
            if (0 == strcmp("add", name))
            {
                puserdata->entered = THE_ADD;
                puserdata->add_i = -1;
                return 1;
            }
        default:
            break;
    }
    return 0;
}

static int next(char **pout, unsigned int *plen, void *userdata)
{
    add_html_userdata *puserdata = (add_html_userdata *)userdata;
    switch (puserdata->entered)
    {
        case THE_ADD:
            puserdata->add_i++;
            return 0 == puserdata->add_i;
        default:
            break;
    }
    return 0;
}

static void leave(char **pout, unsigned int *plen, void *userdata)
{
    add_html_userdata *puserdata = (add_html_userdata *)userdata;
    switch (puserdata->entered)
    {
        case THE_ADD:
            puserdata->entered = THE_NULL;
            break;
        default:
            break;
    }
}

static void extract_name(struct http_message *hm, tm_task *task,
    unsigned int *plen)
{
    char *value = 0;
    extract_post_s(hm, "name", &value);
    if (0 != value && 0 != *value)
    {
        tm_task_set_name(task, value);
        (*plen)++;
    }
}

static void extract_description(struct http_message *hm, tm_task *task)
{
    char *value = 0;
    extract_post_s(hm, "description", &value);
    if (0 != value)
    {
        tm_task_set_description(task, value);
    }
}

static void extract_status(struct http_message *hm, tm_task *task)
{
    char *value = 0;
    extract_post_s(hm, "status", &value);
    if (0 != value && 0 != *value)
    {
        tm_task_set_status(task, value);
    }
}

static void extract_query_from_http_form(struct mg_connection *conn, struct http_message *hm)
{
    tm_task *task = 0;
    unsigned int len = 0;

    tm_task_new(&task);

    extract_name(hm, task, &len);
    extract_description(hm, task);
    extract_status(hm, task);

    if (1 != len)
    {
        send_error(conn, hm, 422);
    }
    else if (0 == tm_add_task(taskman, task))
    {
        send_error(conn, hm, 500);
    }
    else
    {
        unsigned long id = -1;
        tm_task_get_id(task, &id);

        char headers[64];
        sprintf(headers, "Location: /task/%lu.html", id);
        send_303(conn, hm, headers);
    }
    tm_task_free(&task);
}

void do_add_html(struct mg_connection *conn, struct http_message *hm)
{
    unsigned int size = 0;
    char *page = 0;
    add_html_userdata userdata;

    userdata.entered = THE_NULL;

    if (0 == mg_vcmp(&(hm->method), "POST"))
    {
        extract_query_from_http_form(conn, hm);
        return;
    }
    else if (0 != mg_vcmp(&(hm->method), "GET"))
    {
        send_error(conn, hm, 405);
        return;
    }

    if (0 != tpl_file_static_index_html(tag,
        enter,
        next,
        leave,
        &userdata, &size, &page))
    {
        send_ok(conn, hm, "Content-Type: text/html\r\n"
            "Cache-Control: no-store", size, page);
        free(page);
    }
    else
    {
        send_error(conn, hm, 500);
    }
}
