/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <stdio.h>
#include <string.h>
#include <signal.h>

#include <tms.h>

#include <static_lib_js.h>
#include <static_other_html.h>
#include <static_other_js.h>

static int is_not_modified(struct http_message *hm, unsigned long date)
{
    for (unsigned i = 0; 0 != hm->header_names[i].len; i++)
    {
        if (0 == mg_vcmp(&(hm->header_names[i]), "If-None-Match"))
        {
            char date_str[64];
            unsigned int date_str_len;
            date_str_len = sprintf(date_str, "%lu", date);
            return date_str_len + 2 == hm->header_values[i].len &&
                '\"' == hm->header_values[i].p[0] &&
                '\"' == hm->header_values[i].p[date_str_len + 1] &&
                0 == strncmp(date_str, hm->header_values[i].p + 1, date_str_len - 1);
        }
    }
    return 0;
}

void do_other_html(struct mg_connection *conn, struct http_message *hm)
{
    int not_modified = is_not_modified(hm, file_static_other_html_date);
    char headers[256];

    if (0 != mg_vcmp(&(hm->method), "GET"))
    {
        send_error(conn, hm, 405);
    }
    else if (1 == not_modified)
    {
        sprintf(headers, "Cache-Control: public, max-age=86400\r\n"
            "ETag: \"%lu\"", file_static_other_html_date);
        send_304(conn, hm, headers);
    }
    else
    {
        sprintf(headers, "Content-Type: text/html\r\n"
            "Cache-Control: public, max-age=86400\r\n"
            "ETag: \"%lu\"", file_static_other_html_date);
        send_ok(conn, hm, headers, file_static_other_html_size,
            file_static_other_html);
    }
}

void do_lib_js(struct mg_connection *conn, struct http_message *hm)
{
    int not_modified = is_not_modified(hm, file_static_lib_js_date);
    char headers[256];

    if (0 != mg_vcmp(&(hm->method), "GET"))
    {
        send_error(conn, hm, 405);
    }
    else if (1 == not_modified)
    {
        sprintf(headers, "Cache-Control: public, max-age=86400\r\n"
            "ETag: \"%lu\"", file_static_lib_js_date);
        send_304(conn, hm, headers);
    }
    else
    {
        sprintf(headers, "Content-Type: application/javascript\r\n"
            "Cache-Control: public, max-age=86400\r\n"
            "ETag: \"%lu\"", file_static_lib_js_date);
        send_ok(conn, hm, headers, file_static_lib_js_size,
            file_static_lib_js);
    }
}

void do_other_js(struct mg_connection *conn, struct http_message *hm)
{
    int not_modified = is_not_modified(hm, file_static_other_js_date);
    char headers[256];

    if (0 != mg_vcmp(&(hm->method), "GET"))
    {
        send_error(conn, hm, 405);
    }
    else if (1 == not_modified)
    {
        sprintf(headers, "Cache-Control: public, max-age=86400\r\n"
            "ETag: \"%lu\"", file_static_other_js_date);
        send_304(conn, hm, headers);
    }
    else
    {
        sprintf(headers, "Content-Type: application/javascript\r\n"
            "Cache-Control: public, max-age=86400\r\n"
            "ETag: \"%lu\"", file_static_other_js_date);
        send_ok(conn, hm, headers, file_static_other_js_size,
            file_static_other_js);
    }
}
