/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include <criterion/criterion.h>
#include <jansson.h>

#include "common.h"

#include <static_index_html.h>

TestSuite(add_html);

typedef struct reply_results reply_results;
enum reply_entered
{
    RE_NULL,
    RE_ADD
};

struct reply_results
{
    enum reply_entered entered;

    int add_i;
};

static int tag(char **pout, unsigned int *plen, void *userdata, char *tag)
{
    reply_results *pres = (reply_results*)userdata;
    switch (pres->entered)
    {
        case RE_NULL:
            if (0 == strcmp("edit-style", tag) ||
                0 == strcmp("error-style", tag) ||
                0 == strcmp("index-style", tag) ||
                0 == strcmp("task-style", tag) ||
                0 == strcmp("warning-style", tag))
            {
                char *tmp = "style='display: none;'";
                return tpl_append(pout, plen, tmp, strlen(tmp));
            }
            else if (0 == strcmp("add-style", tag))
            {
                return 1;
            }
            break;
        default:
            break;
    }
    return 0;
}

static int enter_top(char **pout, unsigned int *plen, void *userdata, char *name)
{
    reply_results *pres = (reply_results*)userdata;
    if (0 == strcmp("add", name))
    {
        pres->entered = RE_ADD;
        pres->add_i = -1;
        return 1;
    }
    return 0;
}

static int enter(char **pout, unsigned int *plen, void *userdata, char *name)
{
    reply_results *pres = (reply_results*)userdata;
    switch (pres->entered)
    {
        case RE_NULL:
            return enter_top(pout, plen, userdata, name);
        default:
            break;
    }
    return 0;
}

static int next(char **pout, unsigned int *plen, void *userdata)
{
    reply_results *pres = (reply_results*)userdata;
    switch (pres->entered)
    {
        case RE_ADD:
            pres->add_i += 1;
            return 0 == pres->add_i;
        default:
            break;
    }
    return 0;
}

static void leave(char **pout, unsigned int *plen, void *userdata)
{
    reply_results *pres = (reply_results*)userdata;
    switch (pres->entered)
    {
        case RE_ADD:
            pres->entered = RE_NULL;
            break;
        default:
            break;
    }
}

Test(add_html, get)
{
    char *body = 0;
    unsigned int len = 0;
    reply_results res;
    unsigned int port = -1;
    char url[256];
    char len_str[64];
    char *expected_headers[6] =
    {
        "Content-Length", len_str,
        "Content-Type", "text/html",
        "Cache-Control", "no-store"
    };

    res.entered = RE_NULL;

    expect_process_started(0, &port);

    build_body_from_template(tpl_file_static_index_html,
        tag, enter, next, leave, &res, &body, &len);
    sprintf(len_str, "%u", len);
    sprintf(url, "http://%s:%u/add.html", "127.0.0.1", port);
    expect_http_get(url, 0, 200, 3, expected_headers, body);

    free(body);
    expect_process_died(0);
}

Test(add_html, post)
{
    unsigned int port = -1;
    char url[256];
    char *expected_headers[2] =
    {
        "Location", "/task/1.html",
    };

    expect_process_started(0, &port);
    sprintf(url, "http://%s:%u/add.html", "127.0.0.1", port);
    expect_http_post(url, 0,
        "name=Buy%20milk&"
        "description=Goat%20milk&"
        "status=new",
        303, 1, expected_headers, 0);
    expect_process_died(0);
}

Test(add_html, post_minimal)
{
    unsigned int port = -1;
    char url[256];
    char *expected_headers[2] =
    {
        "Location", "/task/1.html",
    };

    expect_process_started(0, &port);
    sprintf(url, "http://%s:%u/add.html", "127.0.0.1", port);
    expect_http_post(url, 0,
        "name=Buy%20milk",
        303, 1, expected_headers, 0);
    expect_process_died(0);
}
