/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <criterion/criterion.h>
#include <jansson.h>

#include <static_lib_js.h>
#include <static_other_js.h>
#include <static_other_html.h>

#include "common.h"

TestSuite(static);

Test(static, lib_js)
{
    unsigned int port = -1;
    char url[256];
    char len_str[64];
    char etag_str[66];
    char *expected_headers[8] =
    {
        "ETag", etag_str,
        "Content-Length", len_str,
        "Content-Type", "application/javascript",
        "Cache-Control", "public, max-age=86400"
    };

    sprintf(len_str, "%lu", file_static_lib_js_size);
    sprintf(etag_str, "\"%lu\"", file_static_lib_js_date);

    expect_process_started(0, &port);
    sprintf(url, "http://%s:%u/lib.js", "127.0.0.1", port);
    expect_http_get(url, 0, 200, 4, expected_headers, file_static_lib_js);
    expect_process_died(0);
}

Test(static, lib_js_304)
{
    unsigned int port = -1;
    char url[256];
    char headers[256];
    char etag_str[66];
    char *expected_headers[4] =
    {
        "ETag", etag_str,
        "Cache-Control", "public, max-age=86400"
    };

    sprintf(etag_str, "\"%lu\"", file_static_lib_js_date);
    sprintf(headers, "If-None-Match: %s\r\n", etag_str);

    expect_process_started(0, &port);
    sprintf(url, "http://%s:%u/lib.js", "127.0.0.1", port);
    expect_http_get(url, headers, 304, 2, expected_headers, 0);
    expect_process_died(0);
}

Test(static, other_js)
{
    unsigned int port = -1;
    char url[256];
    char len_str[64];
    char etag_str[66];
    char *expected_headers[8] =
    {
        "ETag", etag_str,
        "Content-Length", len_str,
        "Content-Type", "application/javascript",
        "Cache-Control", "public, max-age=86400"
    };

    sprintf(len_str, "%lu", file_static_other_js_size);
    sprintf(etag_str, "\"%lu\"", file_static_other_js_date);

    expect_process_started(0, &port);
    sprintf(url, "http://%s:%u/other.js", "127.0.0.1", port);
    expect_http_get(url, 0, 200, 4, expected_headers, file_static_other_js);
    expect_process_died(0);
}

Test(static, other_js_304)
{
    unsigned int port = -1;
    char url[256];
    char etag_str[66];
    char headers[256];
    char *expected_headers[4] =
    {
        "ETag", etag_str,
        "Cache-Control", "public, max-age=86400"
    };

    sprintf(etag_str, "\"%lu\"", file_static_other_js_date);
    sprintf(headers, "If-None-Match: %s\r\n", etag_str);

    expect_process_started(0, &port);
    sprintf(url, "http://%s:%u/other.js", "127.0.0.1", port);
    expect_http_get(url, headers, 304, 2, expected_headers, 0);
    expect_process_died(0);
}

Test(static, other_html)
{
    unsigned int port = -1;
    char url[256];
    char len_str[64];
    char etag_str[66];
    char *expected_headers[8] =
    {
        "ETag", etag_str,
        "Content-Length", len_str,
        "Content-Type", "text/html",
        "Cache-Control", "public, max-age=86400"
    };

    sprintf(len_str, "%lu", file_static_other_html_size);
    sprintf(etag_str, "\"%lu\"", file_static_other_html_date);

    expect_process_started(0, &port);
    sprintf(url, "http://%s:%u/other.html", "127.0.0.1", port);
    expect_http_get(url, 0, 200, 4, expected_headers, file_static_other_html);
    expect_process_died(0);
}

Test(static, other_html_304)
{
    unsigned int port = -1;
    char url[256];
    char etag_str[66];
    char headers[256];
    char *expected_headers[4] =
    {
        "ETag", etag_str,
        "Cache-Control", "public, max-age=86400"
    };

    sprintf(etag_str, "\"%lu\"", file_static_other_html_date);
    sprintf(headers, "If-None-Match: %s\r\n", etag_str);

    expect_process_started(0, &port);
    sprintf(url, "http://%s:%u/other.html", "127.0.0.1", port);
    expect_http_get(url, headers, 304, 2, expected_headers, 0);
    expect_process_died(0);
}
