/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <signal.h>

#include <criterion/criterion.h>
#include <jansson.h>

#include "common.h"

TestSuite(usage);

Test(usage, help_long)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "--help";
    argv[2] = 0;

    expect_process_usage(argv, 0);
}

Test(usage, help_short)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "-h";
    argv[2] = 0;

    expect_process_usage(argv, 0);
}

Test(usage, ip_short_unexpected_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "-i";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}
Test(usage, ip_long_unexpected_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "--ip";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, port_short_unfinished_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "-p";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, port_long_unfinished_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "--port";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, dbfile_short_unfinished_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "-d";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, dbfile_long_unfinished_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "--dbfile";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, unknown_option)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "unknown";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, starts)
{
    char *argv[2];

    argv[0] = "./tms";
    argv[1] = 0;

    expect_process_started_and_died(argv);
}

Test(usage, starts_custom_ip_port_db)
{
    char *argv[8];

    argv[0] = "./tms";
    argv[1] = "-i";
    argv[2] = "0.0.0.0";
    argv[3] = "-p";
    argv[4] = "0";
    argv[5] = "-d";
    argv[6] = "aaa.tm.db";
    argv[7] = 0;

    expect_process_started_and_died(argv);
}

Test(usage, starts_accesslog)
{
    json_t *ret_json = json_object();
    json_t *tmp = 0;
    char *ret_str = 0;
    unsigned int port = -1;
    char url[256];
    char ret_str_len_str[64];
    char *expected_headers[] =
    {
        "Content-Length", ret_str_len_str,
        "Content-Type", "application/json"
    };

    tmp = json_array();
    json_object_set(ret_json, "columns", tmp);
    json_decref(tmp);
    tmp = json_array();
    json_object_set(ret_json, "items", tmp);
    json_decref(tmp);
    ret_str = json_dumps(ret_json, JSON_COMPACT);
    json_decref(ret_json);
    sprintf(ret_str_len_str, "%lu", strlen(ret_str));

    expect_process_started_accesslog(0, &port);

    sprintf(url, "http://%s:%u/api/ping", "127.0.0.1", port);
    expect_http_get(url, 0, 200, 0, 0, 0);
    expect_process_output_accesslog("127.0.0.1", "GET", "/api/ping", 200, 0);

    sprintf(url, "http://%s:%u/api/query", "127.0.0.1", port);
    expect_http_post(url, "Content-Type: application/json\r\n"
        "Accept: application/json\r\n", "{}", 200, 2, expected_headers, ret_str);
    expect_process_output_accesslog("127.0.0.1", "POST", "/api/query", 200, strlen(ret_str));
    free(ret_str);

    sprintf(url, "http://%s:%u/unknown", "127.0.0.1", port);
    expect_http_get(url, 0, 404, 0, 0, 0);
    expect_process_output_accesslog("127.0.0.1", "GET", "/unknown", 404, 0);

    expect_process_died(0);
}
